@@ -14,6 +14,7 @@ from __future__ import division, print_function, absolute_import
 import codecs
 import copy
 from datetime import timedelta, datetime
+from difflib import get_close_matches
 from importlib import import_module
 import re
 import os
@@ -32,11 +33,11 @@ from .docs_resolv import embed_code_links
 from .downloads import generate_zipfiles
 from .sorting import NumberOfCodeLinesSortKey
 from .binder import copy_binder_files, check_binder_conf
-from .directives import MiniGallery
+from .directives import MiniGallery, ImageSg, imagesg_addnode
 
 
-_KNOWN_CSS = ('gallery', 'gallery-binder', 'gallery-dataframe',
-              'gallery-rendered-html')
+_KNOWN_CSS = ('sg_gallery', 'sg_gallery-binder', 'sg_gallery-dataframe',
+              'sg_gallery-rendered-html')
 
 
 class DefaultResetArgv:
@@ -68,6 +69,7 @@ DEFAULT_GALLERY_CONF = {
     'plot_gallery': 'True',
     'download_all_examples': True,
     'abort_on_example_error': False,
+    'only_warn_on_example_error': False,
     'failing_examples': {},
     'passing_examples': [],
     'stale_examples': [],  # ones that did not need to be run due to md5sum
@@ -91,6 +93,9 @@ DEFAULT_GALLERY_CONF = {
     'inspect_global_variables': True,
     'css': _KNOWN_CSS,
     'matplotlib_animations': False,
+    'image_srcset': [],
+    'default_thumb_file': None,
+    'line_numbers': False,
 }
 
 logger = sphinx_compatibility.getLogger('sphinx-gallery')
@@ -105,7 +110,7 @@ def _bool_eval(x):
     return bool(x)
 
 
-def parse_config(app):
+def parse_config(app, check_keys=True):
     """Process the Sphinx Gallery configuration."""
     plot_gallery = _bool_eval(app.builder.config.plot_gallery)
     src_dir = app.builder.srcdir
@@ -114,7 +119,8 @@ def parse_config(app):
     lang = app.builder.config.highlight_language
     gallery_conf = _complete_gallery_conf(
         app.config.sphinx_gallery_conf, src_dir, plot_gallery,
-        abort_on_example_error, lang, app.builder.name, app)
+        abort_on_example_error, lang, app.builder.name, app,
+        check_keys)
 
     # this assures I can call the config in other places
     app.config.sphinx_gallery_conf = gallery_conf
@@ -124,8 +130,21 @@ def parse_config(app):
 
 def _complete_gallery_conf(sphinx_gallery_conf, src_dir, plot_gallery,
                            abort_on_example_error, lang='python',
-                           builder_name='html', app=None):
+                           builder_name='html', app=None, check_keys=True):
     gallery_conf = copy.deepcopy(DEFAULT_GALLERY_CONF)
+    options = sorted(gallery_conf)
+    extra_keys = sorted(set(sphinx_gallery_conf) - set(options))
+    if extra_keys and check_keys:
+        msg = 'Unknown key(s) in sphinx_gallery_conf:\n'
+        for key in extra_keys:
+            options = get_close_matches(key, options, cutoff=0.66)
+            msg += repr(key)
+            if len(options) == 1:
+                msg += ', did you mean %r?' % (options[0],)
+            elif len(options) > 1:
+                msg += ', did you mean one of %r?' % (options,)
+            msg += '\n'
+        raise ConfigError(msg.strip())
     gallery_conf.update(sphinx_gallery_conf)
     if sphinx_gallery_conf.get('find_mayavi_figures', False):
         logger.warning(
@@ -143,20 +162,6 @@ def _complete_gallery_conf(sphinx_gallery_conf, src_dir, plot_gallery,
     gallery_conf['src_dir'] = src_dir
     gallery_conf['app'] = app
 
-    if gallery_conf.get("mod_example_dir", False):
-        backreferences_warning = """\n========
-        Sphinx-Gallery found the configuration key 'mod_example_dir'. This
-        is deprecated, and you should now use the key 'backreferences_dir'
-        instead. Support for 'mod_example_dir' will be removed in a subsequent
-        version of Sphinx-Gallery. For more details, see the backreferences
-        documentation:
-
-        https://sphinx-gallery.github.io/configuration.html#references-to-examples"""  # noqa: E501
-        gallery_conf['backreferences_dir'] = gallery_conf['mod_example_dir']
-        logger.warning(
-            backreferences_warning,
-            type=DeprecationWarning)
-
     # Check capture_repr
     capture_repr = gallery_conf['capture_repr']
     supported_reprs = ['__repr__', '__str__', '_repr_html_']
@@ -709,10 +714,13 @@ def summarize_failing_examples(app, exception):
                 color='brown')
 
     if fail_msgs:
-        raise ExtensionError(
-            "Here is a summary of the problems encountered "
-            "when running the examples\n\n" + "\n".join(fail_msgs) +
-            "\n" + "-" * 79)
+        fail_message = ("Here is a summary of the problems encountered "
+                        "when running the examples\n\n" +
+                        "\n".join(fail_msgs) + "\n" + "-" * 79)
+        if gallery_conf['only_warn_on_example_error']:
+            logger.warning(fail_message)
+        else:
+            raise ExtensionError(fail_message)
 
 
 def collect_gallery_files(examples_dirs, gallery_conf):
@@ -778,6 +786,9 @@ def setup(app):
 
     # Add the custom directive
     app.add_directive('minigallery', MiniGallery)
+    app.add_directive("image-sg", ImageSg)
+
+    imagesg_addnode(app)
 
     app.connect('builder-inited', generate_gallery_rst)
     app.connect('build-finished', copy_binder_files)
