@@ -37,7 +37,8 @@ CONTENT = [
     'And this is a second paragraph',
     '"""',
     '',
-    '# sphinx_gallery_thumbnail_number = 1'
+    '# sphinx_gallery_thumbnail_number = 1',
+    '# sphinx_gallery_defer_figures',
     '# and now comes the module code',
     'import logging',
     'import sys',
@@ -445,10 +446,32 @@ def test_remove_config_comments(gallery_conf, req_pil):
     """Test the gallery_conf['remove_config_comments'] setting."""
     rst = _generate_rst(gallery_conf, 'test.py', CONTENT)
     assert '# sphinx_gallery_thumbnail_number = 1' in rst
+    assert '# sphinx_gallery_defer_figures' in rst
 
     gallery_conf['remove_config_comments'] = True
     rst = _generate_rst(gallery_conf, 'test.py', CONTENT)
     assert '# sphinx_gallery_thumbnail_number = 1' not in rst
+    assert '# sphinx_gallery_defer_figures' not in rst
+
+
+def test_dummy_image_error(gallery_conf, req_pil):
+    """Test correct error is raised if int not provided to
+    sphinx_gallery_dummy_images."""
+    content_image = CONTENT + ["# sphinx_gallery_dummy_images=False", ]
+    msg = "sphinx_gallery_dummy_images setting is not a number"
+    with pytest.raises(ExtensionError, match=msg):
+        _generate_rst(gallery_conf, 'test.py', content_image)
+
+
+def test_final_empty_block(gallery_conf, req_pil):
+    """Test empty final block is removed. Empty final block can occur after
+    sole config comment is removed from final block."""
+    content_block = \
+        CONTENT + ['# %%', '', '# sphinx_gallery_line_numbers = True']
+    gallery_conf['remove_config_comments'] = True
+    rst = _generate_rst(gallery_conf, 'test.py', content_block)
+    want = "RuntimeWarning)\n\n\n.. rst-class:: sphx-glr-timing"
+    assert want in rst
 
 
 def test_download_link_note_only_html(gallery_conf, req_pil):
@@ -625,6 +648,26 @@ def test_output_indentation(gallery_conf, script_vars):
     assert output_test_string == test_string.replace(r"\n", "\n")
 
 
+def test_output_no_ansi(gallery_conf, script_vars):
+    """Test ANSI characters are removed.
+
+    See: https://en.wikipedia.org/wiki/ANSI_escape_code
+    """
+    gallery_conf.update(image_scrapers=())
+    compiler = codeop.Compile()
+
+    code = 'print("\033[94m0.25")'
+    code_block = ("code", code, 1)
+    output = sg.execute_code_block(
+        compiler, code_block, None, script_vars, gallery_conf
+    )
+    output_test_string = "\n".join(
+        [line[4:] for line in output.strip().split("\n")[-3:]]
+    )
+
+    assert output_test_string.split('\n')[-1] == "0.25"
+
+
 def test_empty_output_box(gallery_conf, script_vars):
     """Tests that `print(__doc__)` doesn't produce an empty output box."""
     gallery_conf.update(image_scrapers=())
